/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <ctype.h>
#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <sys/file.h>
#include <string.h>

#include "my_stdio.h"
#include "rotator.h"
#include "observ_table.h"

#define STR_ID      0xff

#define SET_AZ      0x80
#define SET_POL     0x81
/* #define SET_AZ_POL  0x82 */ /* not implemented */
#define SET_VEL     0x83
#define SET_CONTROL 0x84

#define GET_AZ      0x85
#define GET_POL     0x86
#define GET_STATUS  0x87
/* #define GET_RESET   0x88 */ /* not implemented */

#define POL_HORIZONTAL  0
#define POL_VERTICAL    1

/* for GET_RESET: */
#define RS_MASK   0x0f
#define RS_NEAR   0x01
#define RS_DC     0x02
#define RS_EERRO  0x04
#define RS_REMOTO 0x08

#define TO_ROTATOR_L(v) (v & 0x7f)
#define TO_ROTATOR_H(v) ((v >> 7) & 0x7f)
#define FROM_ROTATOR(h,l) (((h << 7) | l) & 0x3f)

struct termios oldtio;

int rotator_setup(char *name)
{
	int fd;
	struct termios tio;

#ifdef DEMO_MODE 
	return 2;
#endif
	/* Open Serial Port : 8 bits data, 1 bit stop, No parity, 1200
           baud */
	if ((fd = open(name, O_RDWR | O_NOCTTY)) < 0 ){
		PVERB(PERR, "Cannot open serial port (%s).\n", name);
		return -1;
	}

	tcgetattr(fd,&oldtio); /* save current port settings */
	tcgetattr(fd,&tio);

	//bzero(&tio, sizeof(tio));
	
	tio.c_cflag &= ~CRTSCTS ;
	/* Zero out the baud rate portion of c_cflag */
	tio.c_cflag &= ~CBAUD;
	/* Set baud rate to 1200 */
	tio.c_cflag |= B1200;
	tio.c_cflag &= ~(CSIZE|PARENB);
	tio.c_cflag |= CS8 ;
	/* Ignore modem control lines */
	tio.c_cflag |= CLOCAL ;

	/* Turn of Canonical Processing and echo */
	tio.c_lflag &= ~ICANON;
	tio.c_lflag &= ~ECHO;
	/* Set MIN to one and TIME to zero */
	tio.c_cc[VMIN]  = 1;
	tio.c_cc[VTIME] = 0;

	/* Ignore BREAK condition on input */
	tio.c_iflag |= IGNBRK;
	/* ignore framing errors and parity errors */
	tio.c_iflag |=IGNPAR;
	/* Disable NL to CR mapping */
	tio.c_iflag &= ~ICRNL;
	/* Disable XON/XOFF flow control on output and input */
	tio.c_iflag &= ~(IXON|IXOFF);

	/* now clean the modem line and activate the settings for modem */
	tcflush(fd, TCOFLUSH|TCIFLUSH);
	tcsetattr(fd, TCSANOW, &tio);
	
	//rotator_set_control(fd, CT_AZ_EN | CT_PO_EN | CT_PW_ON);
	
	return fd;
}


int log_bad_packet(void)
{
	SYSLOG(LOG_ERR, "Bad packet from rotator.");

	return -1;
}


ssize_t soft_read_time_out(struct timeval tv, int fd, void *buf, size_t count)
{
	fd_set rfds;
	int fd_max = fd + 1;
	int retval;

	FD_ZERO(&rfds);
	FD_SET(fd, &rfds);
	retval = select(fd_max, &rfds, NULL, NULL, &tv);
	if (retval) {
		if( FD_ISSET(fd, &rfds) ) {
			retval = read(fd, buf, count);
#ifdef DEBUG_MODE
			if (verbose_flag & PINFO) {
				int __i;
				PVERB(PINFO, "# rotator.c read -> ");
				for(__i=0;__i<retval;__i++)
					PVERB(PINFO, "0x%02x ", 
					      *(((u_char *)buf)+__i) );
				PVERB(PINFO, "\n");
			}
#endif
			return retval;
		}
	}

	return -1;
}


int rotator_wait_confitmation(int fd)
{
	struct timeval tv;
	char answer[5];

	tv.tv_sec = ROTATOR_TIME_OUT;
	tv.tv_usec = 0;	
	if( soft_read_time_out(tv, fd, answer, 5) < 0 ) {
		SYSLOG(LOG_WARNING, "azimuth rotator time out.");
	}else{
			/* TODO: No esta definido aun por Esteban */
		return 0;
	}

	return -1;
}


int discard_noise(int fd)
{
	struct timeval tv;
	char noise;

	tv.tv_sec = 0;
	tv.tv_usec = 0;
	while( soft_read_time_out(tv, fd, &noise, 1) > 0); 

	return 0;
}


ssize_t soft_read(int fd, void *buf, size_t count)
{
	struct timeval tv;

	tv.tv_sec = 5;
	tv.tv_usec = 0;

	return soft_read_time_out(tv, fd, buf, count);
}


ssize_t soft_write(int fd, const void *buf, size_t count)
{
#ifdef DEBUG_MODE
	int i;
	if (verbose_flag & PINFO) {
		PVERB(PINFO, "# rotator.c send -> ");
		for(i=0;i<count;i++)
			PVERB(PINFO, "0x%02x ", *(((u_char *)buf)+i) );
		PVERB(PINFO, "\n");
	}
#endif
#ifdef DEMO_MODE
	return count;
#endif
	discard_noise(fd);
	return write(fd, buf, count);

}


int rotator_set_azimuth(int fd, int az)
{
	int const len = 5;
	char answer[len];
	char cmd[len];
	
	cmd[0] = STR_ID;
	cmd[1] = SET_AZ;
	cmd[2] = TO_ROTATOR_H(az);
	cmd[3] = TO_ROTATOR_L(az);
	cmd[4] = cmd[1] ^ cmd[2] ^ cmd[3];
	soft_write(fd, cmd, len);
	soft_read(fd, answer, len);
	if( strncmp( answer, cmd, len) != 0) {
		return log_bad_packet();
	}

	return rotator_wait_confitmation(fd);
}


int rotator_set_polarization(int fd, char pol)
{
	int const len = 4;
	char answer[len];
	char cmd[len];
	char poln;

	if (pol == HORIZONTAL) poln = POL_HORIZONTAL;
	else if (pol == VERTICAL) poln = POL_VERTICAL;
	else return -1;
	
	cmd[0] = STR_ID;
	cmd[1] = SET_POL;
	cmd[2] = poln;
	cmd[3] = cmd[1] ^ cmd[2];
	soft_write(fd, cmd, len);
	soft_read(fd, answer, len);

	if( strncmp( answer, cmd, len) != 0) {
		return log_bad_packet();
	}

	return rotator_wait_confitmation(fd);
}


int rotator_set_velocity(int fd, int vel)
{
	int const len = 4;
	char answer[len];
	char cmd[len];
	
	cmd[0] = STR_ID;
	cmd[1] = SET_VEL;
	cmd[2] = TO_ROTATOR_L(vel);
	cmd[3] = cmd[1] ^ cmd[2];
	soft_write(fd, cmd, len);
	soft_read(fd, answer, len);

	if( strncmp( answer, cmd, len) != 0) {
		return log_bad_packet();
	}

	return 0;
}



int rotator_get_azimuth(int fd, int *az)
{
	int const len = 5;
	char answer[len];
	char cmd[3];
	
	cmd[0] = STR_ID;
	cmd[1] = GET_AZ;
	cmd[2] = GET_AZ;
	soft_write(fd, cmd, 3);
	soft_read(fd, answer, len);
	if( (answer[1] ^ answer[2] ^ answer[3]) != answer[4] ) {
		return log_bad_packet();
	}
	*az = FROM_ROTATOR(answer[2], answer[3]);

	return 0;
}



int rotator_get_polarization(int fd, char *pol)
{
	int const len = 4;
	char answer[len];
	char cmd[3];
	
	cmd[0] = STR_ID;
	cmd[1] = GET_POL;
	cmd[2] = GET_POL;
	soft_write(fd, cmd, 3);
	soft_read(fd, answer, len);

	if( (answer[1] ^ answer[2]) != answer[3] ) {
		return log_bad_packet();
	}
	if(answer[2] == POL_HORIZONTAL)
		*pol = HORIZONTAL;
	else
		*pol = VERTICAL;

	return 0;
}


int rotator_set_control(int fd, char ctrl)
{
	int const len = 4;
	char answer[len];
	char cmd[len];
	
	cmd[0] = STR_ID;
	cmd[1] = SET_CONTROL;
	cmd[2] = ctrl & CT_MASK;
	cmd[3] = cmd[1] ^ cmd[2];
	soft_write(fd, cmd, len);
	soft_read(fd, answer, len);

	if( strncmp( answer, cmd, len) != 0) {
		return -1;
	}

	return 0;
}


int rotator_get_status(int fd, char *status)
{
	int const len = 4;
	char answer[len];
	char cmd[3];
	
	cmd[0] = STR_ID;
	cmd[1] = GET_STATUS;
	cmd[2] = GET_STATUS;
	soft_write(fd, cmd, 3);
	soft_read(fd, answer, len);
	
	if( (answer[1] ^ answer[2]) != answer[3] ) {
		return log_bad_packet();
	}
	*status = answer[2];

	return 0;
}



int rotator_close(int fd)
{
	//rotator_set_azimuth(fd, ROTATOR_AZ_PARK);
	//rotator_set_control(fd, 0x00);
	tcsetattr(fd,TCSANOW,&oldtio);
	close(fd);

	return 0;
}


