/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

/**
 * At this moment the rf switch port is implemented which ioperm(); 
 **/

#define _GNU_SOURCE

#include <stdio.h>
#include <sys/io.h>

#define SWITCH_1  0x01
#define SWITCH_2  0x02
#define SWITCH_3  0x04
#define CAL_DIODE 0x08


#define ANTENNA_1     (0)
#define ANTENNA_1_CAL (ANTENNA_1|CAL_DIODE)
#define ANTENNA_2     (SWITCH_1)
#define ANTENNA_2_CAL (ANTENNA_2|CAL_DIODE)
#define ANTENNA_3     (SWITCH_2)
#define ANTENNA_3_CAL (ANTENNA_3|CAL_DIODE)
#define ANTENNA_4     (SWITCH_3)
#define ANTENNA_4_CAL (ANTENNA_4|CAL_DIODE)


static int io_perm = 0;
static int as_port;

int antenna_switchs_open(int port)
{
	as_port = port;
	/* Get access to the ports */
	if (ioperm(as_port, 1, 1)) {
		perror("ioperm"); 
		return -1;
	}
	io_perm = 1;

	return 0;
}


int antenna_switchs_close(void)
{
	/* switch off rf sw */
	if (io_perm) 
		outb(0, as_port);
	/* Release access to the ports */
	if (ioperm(as_port, 1, 0)) {
		perror("ioperm"); 
		return -1;
	}
	io_perm = 0;

	return 0;
}


int antenna_switchs_set(char switchs)
{
	
	if (!io_perm) return -1;

	switch (switchs) {
	case 'A': outb(ANTENNA_1,     as_port); break;
	case 'B': outb(ANTENNA_1_CAL, as_port); break;
	case 'C': outb(ANTENNA_2,     as_port); break;
	case 'D': outb(ANTENNA_2_CAL, as_port); break;
	case 'E': outb(ANTENNA_3,     as_port); break;
	case 'F': outb(ANTENNA_3_CAL, as_port); break;
	case 'G': outb(ANTENNA_4,     as_port); break;
	case 'H': outb(ANTENNA_4_CAL, as_port); break;
	default:
		return -1;
	}

	return 0;
}
